@@ -11,6 +11,7 @@ from django.contrib.auth.password_validation import validate_password
 from django.core import exceptions
 from django.core.management.base import BaseCommand, CommandError
 from django.db import DEFAULT_DB_ALIAS
+from django.utils.functional import cached_property
 from django.utils.text import capfirst
 
 
@@ -277,9 +278,21 @@ class Command(BaseCommand):
             else "",
         )
 
+    @cached_property
+    def username_is_unique(self):
+        if self.username_field.unique:
+            return True
+        for unique_constraint in self.UserModel._meta.total_unique_constraints:
+            if (
+                len(unique_constraint.fields) == 1
+                and unique_constraint.fields[0] == self.username_field.name
+            ):
+                return True
+        return False
+
     def _validate_username(self, username, verbose_field_name, database):
         """Validate username. If invalid, return a string error message."""
-        if self.username_field.unique:
+        if self.username_is_unique:
             try:
                 self.UserModel._default_manager.db_manager(database).get_by_natural_key(
                     username
