@@ -6,8 +6,14 @@ from django.contrib.gis.geos import GEOSGeometry
 from django.core.exceptions import FieldError
 from django.db import NotSupportedError
 from django.db.models import (
-    BinaryField, BooleanField, FloatField, Func, IntegerField, TextField,
-    Transform, Value,
+    BinaryField,
+    BooleanField,
+    FloatField,
+    Func,
+    IntegerField,
+    TextField,
+    Transform,
+    Value,
 )
 from django.db.models.functions import Cast
 from django.utils.functional import cached_property
@@ -32,12 +38,21 @@ class GeoFuncMixin:
             except FieldError:
                 output_field = None
             geom = expr.value
-            if not isinstance(geom, GEOSGeometry) or output_field and not isinstance(output_field, GeometryField):
-                raise TypeError("%s function requires a geometric argument in position %d." % (self.name, pos + 1))
+            if (
+                not isinstance(geom, GEOSGeometry)
+                or output_field
+                and not isinstance(output_field, GeometryField)
+            ):
+                raise TypeError(
+                    "%s function requires a geometric argument in position %d."
+                    % (self.name, pos + 1)
+                )
             if not geom.srid and not output_field:
                 raise ValueError("SRID is required for all geometries.")
             if not output_field:
-                self.source_expressions[pos] = Value(geom, output_field=GeometryField(srid=geom.srid))
+                self.source_expressions[pos] = Value(
+                    geom, output_field=GeometryField(srid=geom.srid)
+                )
 
     @property
     def name(self):
@@ -61,8 +76,11 @@ class GeoFuncMixin:
             field = source_fields[pos]
             if not isinstance(field, GeometryField):
                 raise TypeError(
-                    "%s function requires a GeometryField in position %s, got %s." % (
-                        self.name, pos + 1, type(field).__name__,
+                    "%s function requires a GeometryField in position %s, got %s."
+                    % (
+                        self.name,
+                        pos + 1,
+                        type(field).__name__,
                     )
                 )
 
@@ -72,15 +90,17 @@ class GeoFuncMixin:
             expr_srid = expr.output_field.srid
             if expr_srid != base_srid:
                 # Automatic SRID conversion so objects are comparable.
-                res.source_expressions[pos] = Transform(expr, base_srid).resolve_expression(*args, **kwargs)
+                res.source_expressions[pos] = Transform(
+                    expr, base_srid
+                ).resolve_expression(*args, **kwargs)
         return res
 
-    def _handle_param(self, value, param_name='', check_types=None):
-        if not hasattr(value, 'resolve_expression'):
+    def _handle_param(self, value, param_name="", check_types=None):
+        if not hasattr(value, "resolve_expression"):
             if check_types and not isinstance(value, check_types):
                 raise TypeError(
-                    "The %s parameter has the wrong type: should be %s." % (
-                        param_name, check_types)
+                    "The %s parameter has the wrong type: should be %s."
+                    % (param_name, check_types)
                 )
         return value
 
@@ -100,22 +120,31 @@ class SQLiteDecimalToFloatMixin:
     By default, Decimal values are converted to str by the SQLite backend, which
     is not acceptable by the GIS functions expecting numeric values.
     """
+
     def as_sqlite(self, compiler, connection, **extra_context):
-        for expr in self.get_source_expressions():
-            if hasattr(expr, 'value') and isinstance(expr.value, Decimal):
-                expr.value = float(expr.value)
-        return super().as_sql(compiler, connection, **extra_context)
+        copy = self.copy()
+        copy.set_source_expressions(
+            [
+                Value(float(expr.value))
+                if hasattr(expr, "value") and isinstance(expr.value, Decimal)
+                else expr
+                for expr in copy.get_source_expressions()
+            ]
+        )
+        return copy.as_sql(compiler, connection, **extra_context)
 
 
 class OracleToleranceMixin:
     tolerance = 0.05
 
     def as_oracle(self, compiler, connection, **extra_context):
-        tolerance = Value(self._handle_param(
-            self.extra.get('tolerance', self.tolerance),
-            'tolerance',
-            NUMERIC_TYPES,
-        ))
+        tolerance = Value(
+            self._handle_param(
+                self.extra.get("tolerance", self.tolerance),
+                "tolerance",
+                NUMERIC_TYPES,
+            )
+        )
         clone = self.copy()
         clone.set_source_expressions([*self.get_source_expressions(), tolerance])
         return clone.as_sql(compiler, connection, **extra_context)
@@ -129,14 +158,18 @@ class Area(OracleToleranceMixin, GeoFunc):
         return AreaField(self.geo_field)
 
     def as_sql(self, compiler, connection, **extra_context):
-        if not connection.features.supports_area_geodetic and self.geo_field.geodetic(connection):
-            raise NotSupportedError('Area on geodetic coordinate systems not supported.')
+        if not connection.features.supports_area_geodetic and self.geo_field.geodetic(
+            connection
+        ):
+            raise NotSupportedError(
+                "Area on geodetic coordinate systems not supported."
+            )
         return super().as_sql(compiler, connection, **extra_context)
 
     def as_sqlite(self, compiler, connection, **extra_context):
         if self.geo_field.geodetic(connection):
-            extra_context['template'] = '%(function)s(%(expressions)s, %(spheroid)d)'
-            extra_context['spheroid'] = True
+            extra_context["template"] = "%(function)s(%(expressions)s, %(spheroid)d)"
+            extra_context["spheroid"] = True
         return self.as_sql(compiler, connection, **extra_context)
 
 
@@ -152,7 +185,7 @@ class AsGeoJSON(GeoFunc):
     def __init__(self, expression, bbox=False, crs=False, precision=8, **extra):
         expressions = [expression]
         if precision is not None:
-            expressions.append(self._handle_param(precision, 'precision', int))
+            expressions.append(self._handle_param(precision, "precision", int))
         options = 0
         if crs and bbox:
             options = 3
@@ -178,7 +211,7 @@ class AsGML(GeoFunc):
     def __init__(self, expression, version=2, precision=8, **extra):
         expressions = [version, expression]
         if precision is not None:
-            expressions.append(self._handle_param(precision, 'precision', int))
+            expressions.append(self._handle_param(precision, "precision", int))
         super().__init__(*expressions, **extra)
 
     def as_oracle(self, compiler, connection, **extra_context):
@@ -186,7 +219,11 @@ class AsGML(GeoFunc):
         version = source_expressions[0]
         clone = self.copy()
         clone.set_source_expressions([source_expressions[1]])
-        extra_context['function'] = 'SDO_UTIL.TO_GML311GEOMETRY' if version.value == 3 else 'SDO_UTIL.TO_GMLGEOMETRY'
+        extra_context["function"] = (
+            "SDO_UTIL.TO_GML311GEOMETRY"
+            if version.value == 3
+            else "SDO_UTIL.TO_GMLGEOMETRY"
+        )
         return super(AsGML, clone).as_sql(compiler, connection, **extra_context)
 
 
@@ -196,7 +233,7 @@ class AsKML(GeoFunc):
     def __init__(self, expression, precision=8, **extra):
         expressions = [expression]
         if precision is not None:
-            expressions.append(self._handle_param(precision, 'precision', int))
+            expressions.append(self._handle_param(precision, "precision", int))
         super().__init__(*expressions, **extra)
 
 
@@ -204,11 +241,13 @@ class AsSVG(GeoFunc):
     output_field = TextField()
 
     def __init__(self, expression, relative=False, precision=8, **extra):
-        relative = relative if hasattr(relative, 'resolve_expression') else int(relative)
+        relative = (
+            relative if hasattr(relative, "resolve_expression") else int(relative)
+        )
         expressions = [
             expression,
             relative,
-            self._handle_param(precision, 'precision', int),
+            self._handle_param(precision, "precision", int),
         ]
         super().__init__(*expressions, **extra)
 
@@ -223,14 +262,16 @@ class AsWKT(GeoFunc):
     arity = 1
 
 
-class BoundingCircle(OracleToleranceMixin, GeoFunc):
+class BoundingCircle(OracleToleranceMixin, GeomOutputGeoFunc):
     def __init__(self, expression, num_seg=48, **extra):
         super().__init__(expression, num_seg, **extra)
 
     def as_oracle(self, compiler, connection, **extra_context):
         clone = self.copy()
         clone.set_source_expressions([self.get_source_expressions()[0]])
-        return super(BoundingCircle, clone).as_oracle(compiler, connection, **extra_context)
+        return super(BoundingCircle, clone).as_oracle(
+            compiler, connection, **extra_context
+        )
 
 
 class Centroid(OracleToleranceMixin, GeomOutputGeoFunc):
@@ -258,7 +299,7 @@ class Distance(DistanceResultMixin, OracleToleranceMixin, GeoFunc):
     def __init__(self, expr1, expr2, spheroid=None, **extra):
         expressions = [expr1, expr2]
         if spheroid is not None:
-            self.spheroid = self._handle_param(spheroid, 'spheroid', bool)
+            self.spheroid = self._handle_param(spheroid, "spheroid", bool)
         super().__init__(*expressions, **extra)
 
     def as_postgresql(self, compiler, connection, **extra_context):
@@ -276,21 +317,29 @@ class Distance(DistanceResultMixin, OracleToleranceMixin, GeoFunc):
                 )
 
         if not geography and self.geo_field.geodetic(connection):
-            # Geometry fields with geodetic (lon/lat) coordinates need special distance functions
+            # Geometry fields with geodetic (lon/lat) coordinates need special
+            # distance functions.
             if self.spheroid:
-                # DistanceSpheroid is more accurate and resource intensive than DistanceSphere
-                function = connection.ops.spatial_function_name('DistanceSpheroid')
+                # DistanceSpheroid is more accurate and resource intensive than
+                # DistanceSphere.
+                function = connection.ops.spatial_function_name("DistanceSpheroid")
                 # Replace boolean param by the real spheroid of the base field
-                clone.source_expressions.append(Value(self.geo_field.spheroid(connection)))
+                clone.source_expressions.append(
+                    Value(self.geo_field.spheroid(connection))
+                )
             else:
-                function = connection.ops.spatial_function_name('DistanceSphere')
-        return super(Distance, clone).as_sql(compiler, connection, function=function, **extra_context)
+                function = connection.ops.spatial_function_name("DistanceSphere")
+        return super(Distance, clone).as_sql(
+            compiler, connection, function=function, **extra_context
+        )
 
     def as_sqlite(self, compiler, connection, **extra_context):
         if self.geo_field.geodetic(connection):
             # SpatiaLite returns NULL instead of zero on geodetic coordinates
-            extra_context['template'] = 'COALESCE(%(function)s(%(expressions)s, %(spheroid)s), 0)'
-            extra_context['spheroid'] = int(bool(self.spheroid))
+            extra_context[
+                "template"
+            ] = "COALESCE(%(function)s(%(expressions)s, %(spheroid)s), 0)"
+            extra_context["spheroid"] = int(bool(self.spheroid))
         return super().as_sql(compiler, connection, **extra_context)
 
 
@@ -308,7 +357,7 @@ class GeoHash(GeoFunc):
     def __init__(self, expression, precision=None, **extra):
         expressions = [expression]
         if precision is not None:
-            expressions.append(self._handle_param(precision, 'precision', int))
+            expressions.append(self._handle_param(precision, "precision", int))
         super().__init__(*expressions, **extra)
 
     def as_mysql(self, compiler, connection, **extra_context):
@@ -322,8 +371,8 @@ class GeoHash(GeoFunc):
 class GeometryDistance(GeoFunc):
     output_field = FloatField()
     arity = 2
-    function = ''
-    arg_joiner = ' <-> '
+    function = ""
+    arg_joiner = " <-> "
     geom_param_pos = (0, 1)
 
 
@@ -334,7 +383,7 @@ class Intersection(OracleToleranceMixin, GeomOutputGeoFunc):
 
 @BaseSpatialField.register_lookup
 class IsValid(OracleToleranceMixin, GeoFuncMixin, Transform):
-    lookup_name = 'isvalid'
+    lookup_name = "isvalid"
     output_field = BooleanField()
 
     def as_oracle(self, compiler, connection, **extra_context):
@@ -348,8 +397,13 @@ class Length(DistanceResultMixin, OracleToleranceMixin, GeoFunc):
         super().__init__(expr1, **extra)
 
     def as_sql(self, compiler, connection, **extra_context):
-        if self.geo_field.geodetic(connection) and not connection.features.supports_length_geodetic:
-            raise NotSupportedError("This backend doesn't support Length on geodetic fields")
+        if (
+            self.geo_field.geodetic(connection)
+            and not connection.features.supports_length_geodetic
+        ):
+            raise NotSupportedError(
+                "This backend doesn't support Length on geodetic fields"
+            )
         return super().as_sql(compiler, connection, **extra_context)
 
     def as_postgresql(self, compiler, connection, **extra_context):
@@ -359,18 +413,20 @@ class Length(DistanceResultMixin, OracleToleranceMixin, GeoFunc):
             clone.source_expressions.append(Value(self.spheroid))
         elif self.geo_field.geodetic(connection):
             # Geometry fields with geodetic (lon/lat) coordinates need length_spheroid
-            function = connection.ops.spatial_function_name('LengthSpheroid')
+            function = connection.ops.spatial_function_name("LengthSpheroid")
             clone.source_expressions.append(Value(self.geo_field.spheroid(connection)))
         else:
             dim = min(f.dim for f in self.get_source_fields() if f)
             if dim > 2:
                 function = connection.ops.length3d
-        return super(Length, clone).as_sql(compiler, connection, function=function, **extra_context)
+        return super(Length, clone).as_sql(
+            compiler, connection, function=function, **extra_context
+        )
 
     def as_sqlite(self, compiler, connection, **extra_context):
         function = None
         if self.geo_field.geodetic(connection):
-            function = 'GeodesicLength' if self.spheroid else 'GreatCircleLength'
+            function = "GeodesicLength" if self.spheroid else "GreatCircleLength"
         return super().as_sql(compiler, connection, function=function, **extra_context)
 
 
@@ -405,7 +461,9 @@ class Perimeter(DistanceResultMixin, OracleToleranceMixin, GeoFunc):
     def as_postgresql(self, compiler, connection, **extra_context):
         function = None
         if self.geo_field.geodetic(connection) and not self.source_is_geography():
-            raise NotSupportedError("ST_Perimeter cannot use a non-projected non-geography field.")
+            raise NotSupportedError(
+                "ST_Perimeter cannot use a non-projected non-geography field."
+            )
         dim = min(f.dim for f in self.get_source_fields())
         if dim > 2:
             function = connection.ops.perimeter3d
@@ -429,11 +487,11 @@ class Scale(SQLiteDecimalToFloatMixin, GeomOutputGeoFunc):
     def __init__(self, expression, x, y, z=0.0, **extra):
         expressions = [
             expression,
-            self._handle_param(x, 'x', NUMERIC_TYPES),
-            self._handle_param(y, 'y', NUMERIC_TYPES),
+            self._handle_param(x, "x", NUMERIC_TYPES),
+            self._handle_param(y, "y", NUMERIC_TYPES),
         ]
         if z != 0.0:
-            expressions.append(self._handle_param(z, 'z', NUMERIC_TYPES))
+            expressions.append(self._handle_param(z, "z", NUMERIC_TYPES))
         super().__init__(*expressions, **extra)
 
 
@@ -443,16 +501,16 @@ class SnapToGrid(SQLiteDecimalToFloatMixin, GeomOutputGeoFunc):
         expressions = [expression]
         if nargs in (1, 2):
             expressions.extend(
-                [self._handle_param(arg, '', NUMERIC_TYPES) for arg in args]
+                [self._handle_param(arg, "", NUMERIC_TYPES) for arg in args]
             )
         elif nargs == 4:
             # Reverse origin and size param ordering
             expressions += [
-                *(self._handle_param(arg, '', NUMERIC_TYPES) for arg in args[2:]),
-                *(self._handle_param(arg, '', NUMERIC_TYPES) for arg in args[0:2]),
+                *(self._handle_param(arg, "", NUMERIC_TYPES) for arg in args[2:]),
+                *(self._handle_param(arg, "", NUMERIC_TYPES) for arg in args[0:2]),
             ]
         else:
-            raise ValueError('Must provide 1, 2, or 4 arguments to `SnapToGrid`.')
+            raise ValueError("Must provide 1, 2, or 4 arguments to `SnapToGrid`.")
         super().__init__(*expressions, **extra)
 
 
@@ -465,10 +523,10 @@ class Transform(GeomOutputGeoFunc):
     def __init__(self, expression, srid, **extra):
         expressions = [
             expression,
-            self._handle_param(srid, 'srid', int),
+            self._handle_param(srid, "srid", int),
         ]
-        if 'output_field' not in extra:
-            extra['output_field'] = GeometryField(srid=srid)
+        if "output_field" not in extra:
+            extra["output_field"] = GeometryField(srid=srid)
         super().__init__(*expressions, **extra)
 
 
