@@ -134,8 +134,8 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     }
 
     # For these data types:
-    # - MySQL < 8.0.13 and MariaDB < 10.2.1 don't accept default values and
-    #   implicitly treat them as nullable
+    # - MySQL < 8.0.13 doesn't accept default values and implicitly treats them
+    #   as nullable
     # - all versions of MySQL and MariaDB don't support full width database
     #   indexes
     _limited_data_types = (
@@ -200,6 +200,9 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     ops_class = DatabaseOperations
     validation_class = DatabaseValidation
 
+    def get_database_version(self):
+        return self.mysql_version
+
     def get_connection_params(self):
         kwargs = {
             "conv": django_conversions,
@@ -251,6 +254,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         return connection
 
     def init_connection_state(self):
+        super().init_connection_state()
         assignments = []
         if self.features.is_sql_auto_is_null_enabled:
             # SQL_AUTO_IS_NULL controls whether an AUTO_INCREMENT column on
@@ -324,12 +328,11 @@ class DatabaseWrapper(BaseDatabaseWrapper):
                 )
                 if not primary_key_column_name:
                     continue
-                key_columns = self.introspection.get_key_columns(cursor, table_name)
-                for (
-                    column_name,
-                    referenced_table_name,
+                relations = self.introspection.get_relations(cursor, table_name)
+                for column_name, (
                     referenced_column_name,
-                ) in key_columns:
+                    referenced_table_name,
+                ) in relations.items():
                     cursor.execute(
                         """
                         SELECT REFERRING.`%s`, REFERRING.`%s` FROM `%s` as REFERRING
