@@ -49,6 +49,13 @@ class SQLCompiler:
         self.klass_info = None
         self._meta_ordering = None
 
+    def __repr__(self):
+        return (
+            f"<{self.__class__.__qualname__} "
+            f"model={self.query.model.__qualname__} "
+            f"connection={self.connection!r} using={self.using!r}>"
+        )
+
     def setup_query(self):
         if all(self.query.alias_refcount[a] == 0 for a in self.query.alias_map):
             self.query.get_initial_alias()
@@ -147,7 +154,10 @@ class SQLCompiler:
         expressions = self.collapse_group_by(expressions, having_group_by)
 
         for expr in expressions:
-            sql, params = self.compile(expr)
+            try:
+                sql, params = self.compile(expr)
+            except EmptyResultSet:
+                continue
             sql, params = expr.select_format(self, sql, params)
             params_hash = make_hashable(params)
             if (sql, params_hash) not in seen:
@@ -303,8 +313,8 @@ class SQLCompiler:
             ordering = self.query.order_by
         elif self.query.order_by:
             ordering = self.query.order_by
-        elif self.query.get_meta().ordering:
-            ordering = self.query.get_meta().ordering
+        elif (meta := self.query.get_meta()) and meta.ordering:
+            ordering = meta.ordering
             self._meta_ordering = ordering
         else:
             ordering = []
@@ -536,8 +546,16 @@ class SQLCompiler:
                         part_sql = "SELECT * FROM ({})".format(part_sql)
                     # Add parentheses when combining with compound query if not
                     # already added for all compound queries.
-                    elif not features.supports_slicing_ordering_in_compound:
+                    elif (
+                        self.query.subquery
+                        or not features.supports_slicing_ordering_in_compound
+                    ):
                         part_sql = "({})".format(part_sql)
+                elif (
+                    self.query.subquery
+                    and features.supports_slicing_ordering_in_compound
+                ):
+                    part_sql = "({})".format(part_sql)
                 parts += ((part_sql, part_args),)
             except EmptyResultSet:
                 # Omit the empty queryset with UNION and with DIFFERENCE if the
@@ -550,7 +568,9 @@ class SQLCompiler:
         combinator_sql = self.connection.ops.set_operators[combinator]
         if all and combinator == "union":
             combinator_sql += " ALL"
-        braces = "({})" if features.supports_slicing_ordering_in_compound else "{}"
+        braces = "{}"
+        if not self.query.subquery and features.supports_slicing_ordering_in_compound:
+            braces = "({})"
         sql_parts, args_parts = zip(
             *((braces.format(sql), args) for sql, args in parts)
         )
@@ -633,11 +653,20 @@ class SQLCompiler:
                     params.extend(s_params)
                     out_cols.append(s_sql)
 
-                result += [", ".join(out_cols), "FROM", *from_]
+                result += [", ".join(out_cols)]
+                if from_:
+                    result += ["FROM", *from_]
+                elif self.connection.features.bare_select_suffix:
+                    result += [self.connection.features.bare_select_suffix]
                 params.extend(f_params)
 
                 if self.query.select_for_update and features.has_select_for_update:
-                    if self.connection.get_autocommit():
+                    if (
+                        self.connection.get_autocommit()
+                        # Don't raise an exception when database doesn't
+                        # support transactions, as it's a noop.
+                        and features.supports_transactions
+                    ):
                         raise TransactionManagementError(
                             "select_for_update cannot be used outside of a transaction."
                         )
@@ -784,7 +813,8 @@ class SQLCompiler:
         """
         result = []
         if opts is None:
-            opts = self.query.get_meta()
+            if (opts := self.query.get_meta()) is None:
+                return result
         only_load = self.deferred_to_columns()
         start_alias = start_alias or self.query.get_initial_alias()
         # The 'seen_models' is used to optimize checking the needed parent
@@ -837,7 +867,7 @@ class SQLCompiler:
             targets, alias, _ = self.query.trim_joins(targets, joins, path)
             for target in targets:
                 if name in self.query.annotation_select:
-                    result.append(name)
+                    result.append(self.connection.ops.quote_name(name))
                 else:
                     r, p = self.compile(transform_function(target, alias))
                     result.append(r)
@@ -890,10 +920,15 @@ class SQLCompiler:
                 ):
                     item = item.desc() if descending else item.asc()
                 if isinstance(item, OrderBy):
-                    results.append((item, False))
+                    results.append(
+                        (item.prefix_references(f"{name}{LOOKUP_SEP}"), False)
+                    )
                     continue
                 results.extend(
-                    self.find_ordering_name(item, opts, alias, order, already_seen)
+                    (expr.prefix_references(f"{name}{LOOKUP_SEP}"), is_ref)
+                    for expr, is_ref in self.find_ordering_name(
+                        item, opts, alias, order, already_seen
+                    )
                 )
             return results
         targets, alias, _ = self.query.trim_joins(targets, joins, path)
@@ -989,7 +1024,7 @@ class SQLCompiler:
         if not opts:
             opts = self.query.get_meta()
             root_alias = self.query.get_initial_alias()
-        only_load = self.query.get_loaded_field_names()
+        only_load = self.deferred_to_columns()
 
         # Setup for the case when only particular related fields should be
         # included in the related selection.
@@ -1226,6 +1261,8 @@ class SQLCompiler:
                     for klass_info in klass_info.get("related_klass_infos", [])
                 )
 
+        if not self.klass_info:
+            return []
         result = []
         invalid_names = []
         for name in self.query.select_for_update_of:
@@ -1276,7 +1313,7 @@ class SQLCompiler:
         dictionary.
         """
         columns = {}
-        self.query.deferred_to_data(columns, self.query.get_loaded_field_names_cb)
+        self.query.deferred_to_data(columns)
         return columns
 
     def get_converters(self, expressions):
@@ -1410,9 +1447,8 @@ class SQLCompiler:
         result = list(self.execute_sql())
         # Some backends return 1 item tuples with strings, and others return
         # tuples with integers and strings. Flatten them out into strings.
-        output_formatter = (
-            json.dumps if self.query.explain_info.format == "json" else str
-        )
+        format_ = self.query.explain_info.format
+        output_formatter = json.dumps if format_ and format_.lower() == "json" else str
         for row in result[0]:
             if not isinstance(row, str):
                 yield " ".join(output_formatter(c) for c in row)
@@ -1422,7 +1458,7 @@ class SQLCompiler:
 
 class SQLInsertCompiler(SQLCompiler):
     returning_fields = None
-    returning_params = tuple()
+    returning_params = ()
 
     def field_as_sql(self, field, val):
         """
@@ -1537,7 +1573,7 @@ class SQLInsertCompiler(SQLCompiler):
         qn = self.connection.ops.quote_name
         opts = self.query.get_meta()
         insert_statement = self.connection.ops.insert_statement(
-            ignore_conflicts=self.query.ignore_conflicts
+            on_conflict=self.query.on_conflict,
         )
         result = ["%s %s" % (insert_statement, qn(opts.db_table))]
         fields = self.query.fields or [opts.pk]
@@ -1568,8 +1604,11 @@ class SQLInsertCompiler(SQLCompiler):
 
         placeholder_rows, param_rows = self.assemble_as_sql(fields, value_rows)
 
-        ignore_conflicts_suffix_sql = self.connection.ops.ignore_conflicts_suffix_sql(
-            ignore_conflicts=self.query.ignore_conflicts
+        on_conflict_suffix_sql = self.connection.ops.on_conflict_suffix_sql(
+            fields,
+            self.query.on_conflict,
+            self.query.update_fields,
+            self.query.unique_fields,
         )
         if (
             self.returning_fields
@@ -1583,8 +1622,8 @@ class SQLInsertCompiler(SQLCompiler):
             else:
                 result.append("VALUES (%s)" % ", ".join(placeholder_rows[0]))
                 params = [param_rows[0]]
-            if ignore_conflicts_suffix_sql:
-                result.append(ignore_conflicts_suffix_sql)
+            if on_conflict_suffix_sql:
+                result.append(on_conflict_suffix_sql)
             # Skip empty r_sql to allow subclasses to customize behavior for
             # 3rd party backends. Refs #19096.
             r_sql, self.returning_params = self.connection.ops.return_insert_columns(
@@ -1597,12 +1636,12 @@ class SQLInsertCompiler(SQLCompiler):
 
         if can_bulk:
             result.append(self.connection.ops.bulk_insert_sql(fields, placeholder_rows))
-            if ignore_conflicts_suffix_sql:
-                result.append(ignore_conflicts_suffix_sql)
+            if on_conflict_suffix_sql:
+                result.append(on_conflict_suffix_sql)
             return [(" ".join(result), tuple(p for ps in param_rows for p in ps))]
         else:
-            if ignore_conflicts_suffix_sql:
-                result.append(ignore_conflicts_suffix_sql)
+            if on_conflict_suffix_sql:
+                result.append(on_conflict_suffix_sql)
             return [
                 (" ".join(result + ["VALUES (%s)" % ", ".join(p)]), vals)
                 for p, vals in zip(placeholder_rows, param_rows)
@@ -1814,7 +1853,23 @@ class SQLUpdateCompiler(SQLCompiler):
         query.clear_ordering(force=True)
         query.extra = {}
         query.select = []
-        query.add_fields([query.get_meta().pk.name])
+        meta = query.get_meta()
+        fields = [meta.pk.name]
+        related_ids_index = []
+        for related in self.query.related_updates:
+            if all(
+                path.join_field.primary_key for path in meta.get_path_to_parent(related)
+            ):
+                # If a primary key chain exists to the targeted related update,
+                # then the meta.pk value can be used for it.
+                related_ids_index.append((related, 0))
+            else:
+                # This branch will only be reached when updating a field of an
+                # ancestor that is not part of the primary key chain of a MTI
+                # tree.
+                related_ids_index.append((related, len(fields)))
+                fields.append(related._meta.pk.name)
+        query.add_fields(fields)
         super().pre_sql_setup()
 
         must_pre_select = (
@@ -1829,10 +1884,13 @@ class SQLUpdateCompiler(SQLCompiler):
             # don't want them to change), or the db backend doesn't support
             # selecting from the updating table (e.g. MySQL).
             idents = []
+            related_ids = collections.defaultdict(list)
             for rows in query.get_compiler(self.using).execute_sql(MULTI):
                 idents.extend(r[0] for r in rows)
+                for parent, index in related_ids_index:
+                    related_ids[parent].extend(r[index] for r in rows)
             self.query.add_filter("pk__in", idents)
-            self.query.related_ids = idents
+            self.query.related_ids = related_ids
         else:
             # The fast path. Filters and updates in one query.
             self.query.add_filter("pk__in", query)
