@@ -6,7 +6,6 @@ file upload handlers for processing.
 """
 import base64
 import binascii
-import cgi
 import collections
 import html
 from urllib.parse import unquote
@@ -20,6 +19,7 @@ from django.core.exceptions import (
 from django.core.files.uploadhandler import SkipFile, StopFutureHandlers, StopUpload
 from django.utils.datastructures import MultiValueDict
 from django.utils.encoding import force_str
+from django.utils.regex_helper import _lazy_re_compile
 
 __all__ = ("MultiPartParser", "MultiPartParserError", "InputStreamExhausted")
 
@@ -49,6 +49,8 @@ class MultiPartParser:
     and returns a tuple of ``(MultiValueDict(POST), MultiValueDict(FILES))``.
     """
 
+    boundary_re = _lazy_re_compile(rb"[ -~]{0,200}[!-~]")
+
     def __init__(self, META, input_data, upload_handlers, encoding=None):
         """
         Initialize the MultiPartParser object.
@@ -77,7 +79,7 @@ class MultiPartParser:
                 % force_str(content_type)
             )
         boundary = opts.get("boundary")
-        if not boundary or not cgi.valid_boundary(boundary):
+        if not boundary or not self.boundary_re.fullmatch(boundary):
             raise MultiPartParserError(
                 "Invalid boundary in multipart: %s" % force_str(boundary)
             )
@@ -352,6 +354,8 @@ class MultiPartParser:
         file_name = html.unescape(file_name)
         file_name = file_name.rsplit("/")[-1]
         file_name = file_name.rsplit("\\")[-1]
+        # Remove non-printable characters.
+        file_name = "".join([char for char in file_name if char.isprintable()])
 
         if file_name in {"", ".", ".."}:
             return None
@@ -724,7 +728,7 @@ def parse_header(line):
             name = p[:i].strip().lower().decode("ascii")
             if name.endswith("*"):
                 # Lang/encoding embedded in the value (like "filename*=UTF-8''file.ext")
-                # http://tools.ietf.org/html/rfc2231#section-4
+                # https://tools.ietf.org/html/rfc2231#section-4
                 name = name[:-1]
                 if p.count(b"'") == 2:
                     has_encoding = True
@@ -744,7 +748,7 @@ def _parse_header_params(s):
     while s[:1] == b";":
         s = s[1:]
         end = s.find(b";")
-        while end > 0 and s.count(b'"', 0, end) % 2:
+        while end > 0 and (s.count(b'"', 0, end) - s.count(b'\\"', 0, end)) % 2:
             end = s.find(b";", end + 1)
         if end < 0:
             end = len(s)
