@@ -13,21 +13,8 @@ from django.utils.regex_helper import _lazy_re_compile
 from django.utils.safestring import SafeData, SafeString, mark_safe
 from django.utils.text import normalize_newlines
 
-# Configuration for urlize() function.
-TRAILING_PUNCTUATION_CHARS = ".,:;!"
-WRAPPING_PUNCTUATION = [("(", ")"), ("[", "]")]
 
-# List of possible strings used for bullets in bulleted lists.
-DOTS = ["&middot;", "*", "\u2022", "&#149;", "&bull;", "&#8226;"]
-
-word_split_re = _lazy_re_compile(r"""([\s<>"']+)""")
-simple_url_re = _lazy_re_compile(r"^https?://\[?\w", re.IGNORECASE)
-simple_url_2_re = _lazy_re_compile(
-    r"^www\.|^(?!http)\w[^@]+\.(com|edu|gov|int|mil|net|org)($|/.*)$", re.IGNORECASE
-)
-
-
-@keep_lazy(str, SafeString)
+@keep_lazy(SafeString)
 def escape(text):
     """
     Return the given text with ampersands, quotes and angle brackets encoded
@@ -37,7 +24,7 @@ def escape(text):
     This may result in double-escaping. If this is a concern, use
     conditional_escape() instead.
     """
-    return mark_safe(html.escape(str(text)))
+    return SafeString(html.escape(str(text)))
 
 
 _js_escapes = {
@@ -59,7 +46,7 @@ _js_escapes = {
 _js_escapes.update((ord("%c" % z), "\\u%04X" % z) for z in range(32))
 
 
-@keep_lazy(str, SafeString)
+@keep_lazy(SafeString)
 def escapejs(value):
     """Hex encode characters for use in JavaScript strings."""
     return mark_safe(str(value).translate(_js_escapes))
@@ -72,7 +59,7 @@ _json_script_escapes = {
 }
 
 
-def json_script(value, element_id):
+def json_script(value, element_id=None):
     """
     Escape all the HTML/XML special characters with their unicode escapes, so
     value is safe to be output anywhere except for inside a tag attribute. Wrap
@@ -81,11 +68,13 @@ def json_script(value, element_id):
     from django.core.serializers.json import DjangoJSONEncoder
 
     json_str = json.dumps(value, cls=DjangoJSONEncoder).translate(_json_script_escapes)
-    return format_html(
-        '<script id="{}" type="application/json">{}</script>',
-        element_id,
-        mark_safe(json_str),
-    )
+    if element_id:
+        template = '<script id="{}" type="application/json">{}</script>'
+        args = (element_id, mark_safe(json_str))
+    else:
+        template = '<script type="application/json">{}</script>'
+        args = (mark_safe(json_str),)
+    return format_html(template, *args)
 
 
 def conditional_escape(text):
@@ -235,41 +224,123 @@ def smart_urlquote(url):
     return urlunsplit((scheme, netloc, path, query, fragment))
 
 
-@keep_lazy_text
-def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
+class Urlizer:
     """
     Convert any URLs in text into clickable links.
 
-    Works on http://, https://, www. links, and also on links ending in one of
+    Work on http://, https://, www. links, and also on links ending in one of
     the original seven gTLDs (.com, .edu, .gov, .int, .mil, .net, and .org).
     Links can have trailing punctuation (periods, commas, close-parens) and
     leading punctuation (opening parens) and it'll still do the right thing.
-
-    If trim_url_limit is not None, truncate the URLs in the link text longer
-    than this limit to trim_url_limit - 1 characters and append an ellipsis.
-
-    If nofollow is True, give the links a rel="nofollow" attribute.
-
-    If autoescape is True, autoescape the link text and URLs.
     """
-    safe_input = isinstance(text, SafeData)
 
-    def trim_url(x, limit=trim_url_limit):
+    trailing_punctuation_chars = ".,:;!"
+    wrapping_punctuation = [("(", ")"), ("[", "]")]
+
+    simple_url_re = _lazy_re_compile(r"^https?://\[?\w", re.IGNORECASE)
+    simple_url_2_re = _lazy_re_compile(
+        r"^www\.|^(?!http)\w[^@]+\.(com|edu|gov|int|mil|net|org)($|/.*)$", re.IGNORECASE
+    )
+    word_split_re = _lazy_re_compile(r"""([\s<>"']+)""")
+
+    mailto_template = "mailto:{local}@{domain}"
+    url_template = '<a href="{href}"{attrs}>{url}</a>'
+
+    def __call__(self, text, trim_url_limit=None, nofollow=False, autoescape=False):
+        """
+        If trim_url_limit is not None, truncate the URLs in the link text
+        longer than this limit to trim_url_limit - 1 characters and append an
+        ellipsis.
+
+        If nofollow is True, give the links a rel="nofollow" attribute.
+
+        If autoescape is True, autoescape the link text and URLs.
+        """
+        safe_input = isinstance(text, SafeData)
+
+        words = self.word_split_re.split(str(text))
+        return "".join(
+            [
+                self.handle_word(
+                    word,
+                    safe_input=safe_input,
+                    trim_url_limit=trim_url_limit,
+                    nofollow=nofollow,
+                    autoescape=autoescape,
+                )
+                for word in words
+            ]
+        )
+
+    def handle_word(
+        self,
+        word,
+        *,
+        safe_input,
+        trim_url_limit=None,
+        nofollow=False,
+        autoescape=False,
+    ):
+        if "." in word or "@" in word or ":" in word:
+            # lead: Punctuation trimmed from the beginning of the word.
+            # middle: State of the word.
+            # trail: Punctuation trimmed from the end of the word.
+            lead, middle, trail = self.trim_punctuation(word)
+            # Make URL we want to point to.
+            url = None
+            nofollow_attr = ' rel="nofollow"' if nofollow else ""
+            if self.simple_url_re.match(middle):
+                url = smart_urlquote(html.unescape(middle))
+            elif self.simple_url_2_re.match(middle):
+                url = smart_urlquote("http://%s" % html.unescape(middle))
+            elif ":" not in middle and self.is_email_simple(middle):
+                local, domain = middle.rsplit("@", 1)
+                try:
+                    domain = punycode(domain)
+                except UnicodeError:
+                    return word
+                url = self.mailto_template.format(local=local, domain=domain)
+                nofollow_attr = ""
+            # Make link.
+            if url:
+                trimmed = self.trim_url(middle, limit=trim_url_limit)
+                if autoescape and not safe_input:
+                    lead, trail = escape(lead), escape(trail)
+                    trimmed = escape(trimmed)
+                middle = self.url_template.format(
+                    href=escape(url),
+                    attrs=nofollow_attr,
+                    url=trimmed,
+                )
+                return mark_safe(f"{lead}{middle}{trail}")
+            else:
+                if safe_input:
+                    return mark_safe(word)
+                elif autoescape:
+                    return escape(word)
+        elif safe_input:
+            return mark_safe(word)
+        elif autoescape:
+            return escape(word)
+        return word
+
+    def trim_url(self, x, *, limit):
         if limit is None or len(x) <= limit:
             return x
         return "%s…" % x[: max(0, limit - 1)]
 
-    def trim_punctuation(lead, middle, trail):
+    def trim_punctuation(self, word):
         """
-        Trim trailing and wrapping punctuation from `middle`. Return the items
-        of the new state.
+        Trim trailing and wrapping punctuation from `word`. Return the items of
+        the new state.
         """
+        lead, middle, trail = "", word, ""
         # Continue trimming until middle remains unchanged.
         trimmed_something = True
         while trimmed_something:
             trimmed_something = False
             # Trim wrapping punctuation.
-            for opening, closing in WRAPPING_PUNCTUATION:
+            for opening, closing in self.wrapping_punctuation:
                 if middle.startswith(opening):
                     middle = middle[len(opening) :]
                     lead += opening
@@ -286,7 +357,7 @@ def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
             # as encoded entities contain ';'). Unescape entities to avoid
             # breaking them by removing ';'.
             middle_unescaped = html.unescape(middle)
-            stripped = middle_unescaped.rstrip(TRAILING_PUNCTUATION_CHARS)
+            stripped = middle_unescaped.rstrip(self.trailing_punctuation_chars)
             if middle_unescaped != stripped:
                 punctuation_count = len(middle_unescaped) - len(stripped)
                 trail = middle[-punctuation_count:] + trail
@@ -294,6 +365,7 @@ def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
                 trimmed_something = True
         return lead, middle, trail
 
+    @staticmethod
     def is_email_simple(value):
         """Return True if value looks like an email address."""
         # An @ must be in the middle of the value.
@@ -309,50 +381,15 @@ def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
             return False
         return True
 
-    words = word_split_re.split(str(text))
-    for i, word in enumerate(words):
-        if "." in word or "@" in word or ":" in word:
-            # lead: Current punctuation trimmed from the beginning of the word.
-            # middle: Current state of the word.
-            # trail: Current punctuation trimmed from the end of the word.
-            lead, middle, trail = "", word, ""
-            # Deal with punctuation.
-            lead, middle, trail = trim_punctuation(lead, middle, trail)
 
-            # Make URL we want to point to.
-            url = None
-            nofollow_attr = ' rel="nofollow"' if nofollow else ""
-            if simple_url_re.match(middle):
-                url = smart_urlquote(html.unescape(middle))
-            elif simple_url_2_re.match(middle):
-                url = smart_urlquote("http://%s" % html.unescape(middle))
-            elif ":" not in middle and is_email_simple(middle):
-                local, domain = middle.rsplit("@", 1)
-                try:
-                    domain = punycode(domain)
-                except UnicodeError:
-                    continue
-                url = "mailto:%s@%s" % (local, domain)
-                nofollow_attr = ""
+urlizer = Urlizer()
 
-            # Make link.
-            if url:
-                trimmed = trim_url(middle)
-                if autoescape and not safe_input:
-                    lead, trail = escape(lead), escape(trail)
-                    trimmed = escape(trimmed)
-                middle = '<a href="%s"%s>%s</a>' % (escape(url), nofollow_attr, trimmed)
-                words[i] = mark_safe("%s%s%s" % (lead, middle, trail))
-            else:
-                if safe_input:
-                    words[i] = mark_safe(word)
-                elif autoescape:
-                    words[i] = escape(word)
-        elif safe_input:
-            words[i] = mark_safe(word)
-        elif autoescape:
-            words[i] = escape(word)
-    return "".join(words)
+
+@keep_lazy_text
+def urlize(text, trim_url_limit=None, nofollow=False, autoescape=False):
+    return urlizer(
+        text, trim_url_limit=trim_url_limit, nofollow=nofollow, autoescape=autoescape
+    )
 
 
 def avoid_wrapping(value):
