@@ -1,4 +1,3 @@
-import html.entities
 import re
 import unicodedata
 from gzip import GzipFile
@@ -376,28 +375,6 @@ def smart_split(text):
         yield bit[0]
 
 
-def _replace_entity(match):
-    text = match[1]
-    if text[0] == "#":
-        text = text[1:]
-        try:
-            if text[0] in "xX":
-                c = int(text[1:], 16)
-            else:
-                c = int(text)
-            return chr(c)
-        except ValueError:
-            return match[0]
-    else:
-        try:
-            return chr(html.entities.name2codepoint[text])
-        except KeyError:
-            return match[0]
-
-
-_entity_re = _lazy_re_compile(r"&(#?[xX]?(?:[0-9a-fA-F]+|\w{1,8}));")
-
-
 @keep_lazy_text
 def unescape_string_literal(s):
     r"""
@@ -413,7 +390,7 @@ def unescape_string_literal(s):
         >>> unescape_string_literal("'\'ab\' c'")
         "'ab' c"
     """
-    if s[0] not in "\"'" or s[-1] != s[0]:
+    if not s or s[0] not in "\"'" or s[-1] != s[0]:
         raise ValueError("Not a string literal: %r" % s)
     quote = s[0]
     return s[1:-1].replace(r"\%s" % quote, quote).replace(r"\\", "\\")
